package myapplicationmodulelistenerapp.common;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import java.util.Map;
import java.util.Set;

import oracle.jbo.client.Configuration;
import oracle.jbo.common.ampool.ApplicationPool;
import oracle.jbo.common.ampool.PoolMgr;
import oracle.jbo.common.ampool.SessionCookie;
import oracle.jbo.common.ampool.Statistics;


public class PoolMgrHelper
{
	private static final IdShortener APPLICATION_POOL_ID_SHORTENER =
		new IdShortener("ap", 21);

	protected static PoolMgrHelper fDefaultPoolMgrHelper = new PoolMgrHelper();

	protected PoolMgrHelper()
	{}

	public static PoolMgrHelper getDefaultInstance()
	{
		return fDefaultPoolMgrHelper;
	}

	public static List<ApplicationPool> getAllApplicationPools()
		throws Exception
	{
		List<ApplicationPool> vApplicationPools = new LinkedList<ApplicationPool>();

		PoolMgr vPoolMgr = PoolMgr.getInstance();

		Enumeration vResourcePoolsEnum = vPoolMgr.getResourcePools();
		while (vResourcePoolsEnum.hasMoreElements())
		{
			vApplicationPools.add(
				(ApplicationPool)vResourcePoolsEnum.nextElement());
		}

		return vApplicationPools;
	}

	public String getShortApplicationPoolId(ApplicationPool pApplicationPool)
	{
		return getShortApplicationPoolId(pApplicationPool.getSignature());
	}

	public String getShortApplicationPoolId(long vPoolSignature)
	{
		return APPLICATION_POOL_ID_SHORTENER.getShortId(
			vPoolSignature, getIdShortenerListener());
	}

	public IdShortenerListener getIdShortenerListener()
	{
		return IdShortenerListener.IGNORE_ID_SHORTENER_LISTENER;
	}

	public ApplicationPool findApplicationPool(SessionCookie pSessionCookie)
	{
		long vPoolSignature = pSessionCookie.getPoolSignature();
		PoolMgr vPoolMgr = PoolMgr.getInstance();

		Enumeration vResourcePoolsEnum = vPoolMgr.getResourcePools();
		while (vResourcePoolsEnum.hasMoreElements())
		{
			ApplicationPool vApplicationPool =
				(ApplicationPool)vResourcePoolsEnum.nextElement();
			long vSignature = vApplicationPool.getSignature();
			if (vSignature == vPoolSignature)
			{
				return vApplicationPool;
			}
		}
		throw new RuntimeException("no ApplicationPool for pool signature " + vPoolSignature);
	}

	public String getStatisticsInOneLine(SessionCookie pSessionCookie)
	{
		ApplicationPool vApplicationPool = findApplicationPool(pSessionCookie);
		return getStatisticsInOneLine(vApplicationPool);
	}

	public String getStatisticsInOneLine(ApplicationPool pApplicationPool)
	{
		String vShortApplicationPoolId = getShortApplicationPoolId(pApplicationPool);
		Statistics vStatistics = pApplicationPool.getStatistics();
		return "(" + vShortApplicationPoolId + ")"
			+ getStatisticsInOneLine(vStatistics);
	}

	public String getStatisticsInOneLine(Statistics pStatistics)
	{
		// based on ApplicationPoolLogger.dumpPoolStatistics()
		String vResult = "[";
		vResult = vResult + getAMPoolInstanceLifetimeStats(pStatistics);
		vResult = vResult + "; " + getAMPoolStateManagementStats(pStatistics);
		vResult = vResult + "; " + getAMPoolUseStats(pStatistics);
		vResult = vResult + "; " + getAMPoolInstanceStats(pStatistics);
//		vResult = vResult + "; " + getAMPoolInstanceAgeStats(pStatistics);
		vResult = vResult + "; " + getAMPoolSessionStats(pStatistics);
//		vResult = vResult + "; " + getAMPoolSessionAgeStats(pStatistics);
		return vResult + "]";
	}

	public String getAMPoolInstanceLifetimeStats(Statistics pStatistics)
	{
		return "cre " + pStatistics.mNumOfInstanceCreations // Number of application module creations
			+ "; rem " + pStatistics.mNumOfInstanceRemovals; // Number of application module removals
	}

	public String getAMPoolStateManagementStats(Statistics pStatistics)
	{
		return "act " + pStatistics.mNumOfStateActivations // Number of transactional state activations
			+ "; pas " + pStatistics.mNumOfStatePassivations; // Number of transactional state passivations
	}

	public String getAMPoolUseStats(Statistics pStatistics)
	{
		return "co " + pStatistics.mNumOfCheckouts // Number of application pool check outs
			+ "; ci " + pStatistics.mNumOfCheckins // Number of application pool check ins
			+ "; ref reu " + pStatistics.mNumOfInstancesReused // Number of referenced application modules that were reused
			+ "; ref rec " + pStatistics.mNumOfReferencedInstancesRecycled // Number of referenced application modules that were recycled
			+ "; unr rec " + pStatistics.mNumOfUnreferencedInstancesRecycled // Number of unreferenced application modules that were recycled
			+ "; fail " + pStatistics.mNumOfCheckoutFailures; // Number of application pool check out failures
	}

	public String getAMPoolInstanceStats(Statistics pStatistics)
	{
		return "tot am " + pStatistics.mResourceCount // Total number of application modules in the pool
			+ "; max am " + pStatistics.mMaxNumOfInstances // Maximum number of application modules in the pool
			+ "; avg am " + pStatistics.mAvgNumOfInstances // Average number of application modules in the pool
			+ "; tot ava " + pStatistics.mAvailableResourceCount // Total number of available application modules in the pool
			+ "; avg ava " + pStatistics.mAvgNumOfAvailableInstances // Average number of available application modules in the pool
			+ "; avg una " + pStatistics.mAvgNumOfUnavailableInstances // Average number of unavailable application modules in the pool
			+ "; tot ref " + pStatistics.mReferencedApplicationModules; // Total number of referenced application modules in the pool
	}

	public String getAMPoolInstanceAgeStats(Statistics pStatistics)
	{
		return "ref un 10m" + pStatistics.mRefInstanceAgeHistogram.mBuckets[3] // Number of referenced instances unused for >10 min
			+ "; ref un 5m " + pStatistics.mRefInstanceAgeHistogram.mBuckets[2] // Number of referenced instances unused for >5 min
			+ "; ref un 1m " + pStatistics.mRefInstanceAgeHistogram.mBuckets[1] // Number of referenced instances unused for >1 min
			+ "; ref us 1m " + pStatistics.mRefInstanceAgeHistogram.mBuckets[0] // Number of referenced instances used during last 1 min
			+ "; un 10m " + pStatistics.mUnrefInstanceAgeHistogram.mBuckets[3] // Number of instances unused for >10 min
			+ "; un 5m " + pStatistics.mUnrefInstanceAgeHistogram.mBuckets[2] // Number of instances unused for >5 min
			+ "; un 1m " + pStatistics.mUnrefInstanceAgeHistogram.mBuckets[1] // Number of instances unused for >1 min
			+ "; us 1m " + pStatistics.mUnrefInstanceAgeHistogram.mBuckets[0]; // Number of instances used during last 1 min
	}

	public String getAMPoolSessionStats(Statistics pStatistics)
	{
		return "ses " + pStatistics.mNumOfSessions // Number of sessions registered with the pool
			+ "; avg ses " + pStatistics.mAvgNumOfSessionsReferencingState; // Average number of sessions referencing transactional state
	}

	public String getAMPoolSessionAgeStats(Statistics pStatistics)
	{
		return "ses ia 10m " + pStatistics.mSessionAgeHistogram.mBuckets[3] // Number of sessions inactive for >10 min
			+ "; ses ia 5m " + pStatistics.mSessionAgeHistogram.mBuckets[2] // Number of sessions inactive for >5 min
			+ "; ses ia 1m " + pStatistics.mSessionAgeHistogram.mBuckets[1] // Number of sessions inactive for >1 min
			+ "; ses ac 1m " + pStatistics.mSessionAgeHistogram.mBuckets[0]; // Number of sessions active during last 1 min
	}

	public String getEnvironmentInfo(ApplicationPool pApplicationPool)
	{
		Hashtable vEnvironment = pApplicationPool.getEnvironment();
		Set<Map.Entry> vEntrySet = vEnvironment.entrySet();
		String vInfo = null;
		for (Map.Entry vEntry : vEntrySet)
		{
			Object vKey = vEntry.getKey();
			Object vValue = vEntry.getValue();
			if (Configuration.DB_PASSWORD_PROPERTY.equals(vKey))
			{
				vValue = "!!password-replaced!!";
			}
			vInfo = (vInfo != null ? vInfo + "; " : "") + vKey + " = " + vValue;
		}
		return vInfo;
	}

	public String getJDBCDataSourceName(ApplicationPool pApplicationPool)
	{
		Hashtable vEnvironment = pApplicationPool.getEnvironment();
		return (String)vEnvironment.get(Configuration.JDBC_DS_NAME);
	}
}
