package myapplicationmodulelistenerapp.common;

import java.io.IOException;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.management.JMX;
import javax.management.MBeanServerConnection;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import weblogic.management.runtime.JDBCDataSourceRuntimeMBean;


public class MyMBeanHelper
{
	public static final String JMX_RUNTIME_NAME = "java:comp/env/jmx/runtime";
	public static final String JMX_EDIT_NAME = "java:comp/env/jmx/edit";

	public static final String JDBCDATASOURCERUNTIME_QUERYOBJECTNAME =
		"*:*,Type=JDBCDataSourceRuntime";

	protected MyMBeanHelper()
	{}

	public static MBeanServerConnection lookupMBeanServerConnectionRuntime()
	{
		return lookupMBeanServerConnectionRuntime(getInitialContext());
	}

	public static MBeanServerConnection lookupMBeanServerConnectionRuntime(
		InitialContext pInitialContext)
	{
		return lookupMBeanServerConnection(pInitialContext, JMX_RUNTIME_NAME);
	}

	public static MBeanServerConnection lookupMBeanServerConnectionEdit()
	{
		return lookupMBeanServerConnectionEdit(getInitialContext());
	}

	public static MBeanServerConnection lookupMBeanServerConnectionEdit(
		InitialContext pInitialContext)
	{
		return lookupMBeanServerConnection(pInitialContext, JMX_EDIT_NAME);
	}

	public static MBeanServerConnection lookupMBeanServerConnection(
		InitialContext pInitialContext, String pName)
	{
		if (pInitialContext == null)
		{
			throw new RuntimeException("pInitialContext == null");
		}
		try
		{
			return (MBeanServerConnection)pInitialContext.lookup(pName);
		}
		catch (NamingException ex)
		{
			throw new RuntimeException(
				"unable to lookup name " + pName + ", ex = " + ex, ex);
		}
	}

	public static InitialContext getInitialContext()
	{
		try
		{
			return new InitialContext();
		}
		catch (NamingException ex)
		{
			throw new RuntimeException(
				"unable to create InitialContext, ex = " + ex, ex);
		}
	}

	public static List<String> queryMBeansObjects(
		MBeanServerConnection pMBeanServerConnection, String pQueryObjectName)
	{
		ObjectName vQueryObjectName = createObjectName(pQueryObjectName);
		Set<ObjectInstance> vQueriedObjectInstances = null;
		try
		{
			vQueriedObjectInstances =
				pMBeanServerConnection.queryMBeans(vQueryObjectName, null);
		}
		catch (IOException ex)
		{
			throw new RuntimeException("query MBeans issue for "
				+ vQueryObjectName + ", ex = " + ex, ex);
		}

		List<String> vResult = new ArrayList<String>();
		for (ObjectInstance vObjectInstance : vQueriedObjectInstances)
		{
			vResult.add("" + vObjectInstance.getObjectName());
		}
		return vResult;
	}

	public static ObjectName createObjectName(String pObjectName)
	{
		try
		{
			return new ObjectName(pObjectName);
		}
		catch (MalformedObjectNameException ex)
		{
			throw new RuntimeException("object name issue for "
				+ pObjectName + ", ex = " + ex, ex);
		}
	}

	public static String getFirstJDBCDataSourceRuntimeMBeanName(
		MBeanServerConnection pMBeanServerConnection)
	{
		List<String> vMBeansObjectsNames = queryMBeansObjects(
			pMBeanServerConnection, JDBCDATASOURCERUNTIME_QUERYOBJECTNAME);
		if (vMBeansObjectsNames.size() > 0)
		{
			return vMBeansObjectsNames.get(0);
		}
		return null;
	}

	public static JDBCDataSourceRuntimeMBean getProxyJDBCDataSourceRuntimeMBean(
		MBeanServerConnection pMBeanServerConnection, String pObjectName)
	{
		return getProxyJDBCDataSourceRuntimeMBean(pMBeanServerConnection,
			createObjectName(pObjectName));
	}

	public static JDBCDataSourceRuntimeMBean getProxyJDBCDataSourceRuntimeMBean(
		MBeanServerConnection pMBeanServerConnection, ObjectName pObjectName)
	{
		return JMX.newMBeanProxy(pMBeanServerConnection, pObjectName,
			JDBCDataSourceRuntimeMBean.class);
	}

	public static String getJDBCDataSourceRuntimeMBeanInfo(
		JDBCDataSourceRuntimeMBean pJDBCDataSourceRuntimeMBean)
	{
		return "["
			+ "cur cap " + pJDBCDataSourceRuntimeMBean.getCurrCapacity()
			+ "; avai " + pJDBCDataSourceRuntimeMBean.getNumAvailable()
			+ "; unav " + pJDBCDataSourceRuntimeMBean.getNumUnavailable()
			+ "; cp hig " + pJDBCDataSourceRuntimeMBean.getCurrCapacityHighCount()

			+ "; ac cur " + pJDBCDataSourceRuntimeMBean.getActiveConnectionsCurrentCount()
			+ "; ac avg " + pJDBCDataSourceRuntimeMBean.getActiveConnectionsAverageCount()
			+ "; ac hig " + pJDBCDataSourceRuntimeMBean.getActiveConnectionsHighCount()

//			+ "; cd " + pJDBCDataSourceRuntimeMBean.isCachingDisabled() // javax.management.AttributeNotFoundException: com.bea:ServerRuntime=DefaultServer,Name= ... ,Type=JDBCDataSourceRuntime:CachingDisabled
			+ "; co del " + pJDBCDataSourceRuntimeMBean.getConnectionDelayTime()
			+ "; co tot " + pJDBCDataSourceRuntimeMBean.getConnectionsTotalCount()

			+ "; res req " + pJDBCDataSourceRuntimeMBean.getReserveRequestCount()
			+ "; fai res " + pJDBCDataSourceRuntimeMBean.getFailedReserveRequestCount()
			+ "; fai rec " + pJDBCDataSourceRuntimeMBean.getFailuresToReconnectCount()

			+ "; h avai " + pJDBCDataSourceRuntimeMBean.getHighestNumAvailable()
			+ "; h unav " + pJDBCDataSourceRuntimeMBean.getHighestNumUnavailable()

			+ "; lea co " + pJDBCDataSourceRuntimeMBean.getLeakedConnectionCount()

			+ "; psc acc " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheAccessCount()
			+ "; psc add " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheAddCount()
			+ "; psc csi " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheCurrentSize()
			+ "; psc del " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheDeleteCount()
			+ "; psc hit " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheHitCount()
			+ "; psc mis " + pJDBCDataSourceRuntimeMBean.getPrepStmtCacheMissCount()

			+ "; wfc cur " + pJDBCDataSourceRuntimeMBean.getWaitingForConnectionCurrentCount()
			+ "; wfc fai " + pJDBCDataSourceRuntimeMBean.getWaitingForConnectionFailureTotal()
			+ "; wfc hig " + pJDBCDataSourceRuntimeMBean.getWaitingForConnectionHighCount()
			+ "; wfc tot " + pJDBCDataSourceRuntimeMBean.getWaitingForConnectionTotal()
			+ "; wse hig " + pJDBCDataSourceRuntimeMBean.getWaitSecondsHighCount()
			+ "]";
	}
}
